<?php

namespace Drupal\proc\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\EntityReferenceAutocompleteWidget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\proc\Entity\Proc;

/**
 * Defines the 'proc_entity_reference_widget' field widget.
 *
 * @FieldWidget(
 *   id = "proc_entity_reference_widget",
 *   module = "proc",
 *   label = @Translation("Proc Entity Reference Field Widget"),
 *   field_types = {
 *     "proc_entity_reference_field"
 *   }
 * )
 */
class ProcEntityReferenceWidget extends EntityReferenceAutocompleteWidget {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    // Call the parent formElement method to get the base element.
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    // Get global settings and field name.
    $config = \Drupal::config('proc.settings');
    $proc_field_name = $items->getName();

    // Append the match operation to the selection settings.
    $selection_settings = $this->getFieldSetting('handler_settings') + [
      'match_operator' => $this->getSetting('match_operator'),
      'match_limit' => $this->getSetting('match_limit'),
    ];

    // Extract various settings into local variables.
    $field_input_mode = $selection_settings['proc']['proc_field_input_mode'] ?? '';
    $to_recipients_field_name = $selection_settings['proc']['direct_fetcher']['proc_field_recipients_to_field'] ?? '';
    $carbon_copy_recipients_field_name = $selection_settings['proc']['direct_fetcher']['proc_field_recipients_cc_field'] ?? '';
    $preset_recipients_csv = $selection_settings['proc']['direct_fetcher']['proc_field_recipients_manual_fetcher'] ?? '';
    $fetcher_endpoint = $selection_settings['proc']['proc_field_recipients_fetcher_endpoint'] ?? '';
    $cache_password = $selection_settings['proc']['proc_allow_password_caching'] ?? '0';
    $trigger_format = $selection_settings['proc']['proc_trigger_format'] ?? '0';
    $submit_element_id = $selection_settings['proc']['proc_master_field_submit_id'] ?? '';
    $trigger_fields = $selection_settings['proc']['proc_decryption_trigger_fields'] ?? '0';

    // JavaScript variables.
    $direct_fetcher_js_prefix = <<<JS

      let to_recipients_collection = [];
      let cc_recipients_collection = [];
      let pre_set_recipients_csv = "{$preset_recipients_csv}";
      let fetcher_endpoint = "{$fetcher_endpoint}";
      let recipients = [];
    JS;

    $direct_fetcher_to = !empty($to_recipients_field_name) ? <<<JS

      to_recipients_collection = Object.values(document.querySelectorAll("input[name^='{$to_recipients_field_name}']"));
    JS : '';

    $direct_fetcher_cc = !empty($carbon_copy_recipients_field_name) ? <<<JS

      cc_recipients_collection = Object.values(document.querySelectorAll("input[name^='{$carbon_copy_recipients_field_name}']"));
    JS : '';

    $fetcher_value = <<<JS
      let proc_path = proc_path_prefix_add + "/" + ids_csv + proc_path_suffix;
      let ajaxSettings = {
        url: proc_path,
        dialogType: "dialog",
        dialog: { width: 400, classes: { "ui-dialog": "class-{$proc_field_name}" } },
      };
      let myAjaxObject = Drupal.ajax(ajaxSettings);
      myAjaxObject.execute();
    JS;

    if ($fetcher_endpoint) {
      $fetcher_value = <<<JS
        // Fetcher endpoint CSV:
        let fetcher_endpoint_csv = "";
        let endpoint_response;

        if (fetcher_endpoint != "") {
          endpoint_response = fetch(fetcher_endpoint, {
              method: "get"
          }).then(async function(response) {
            return response.json();
          }).catch(async function(err) {
            console.info("Error.");
          });
        }
        let endpoint_recipients_csv = endpoint_response.then(async function(result) {
          return result.map(item => item.id).join();
        });
        endpoint_recipients_csv.then(function(result) {
          if (result != "") {
            var separator = "";
            if (ids_csv != "") {
              separator = ",";
            }
            ids_csv = ids_csv + separator + result;
            proc_path = proc_path_prefix_add + "/" + ids_csv + proc_path_suffix;

            ajaxSettings = {
              url: proc_path,
              dialogType: "dialog",
              dialog: { width: 400, classes: { "ui-dialog": "class-{$proc_field_name}" } },
            };

            let myAjaxObject = Drupal.ajax(ajaxSettings);
            myAjaxObject.execute();
            return false;
          }
        });
      JS;
    }

    $direct_fetcher_js = <<<JS

      jQuery(this).attr("href", "#");

      // Check if this is an autocomplete deluxe field:
      if (to_recipients_collection.length) {
        if (to_recipients_collection[0].value == "" && to_recipients_collection[1].value.includes('""""')) {
          // This is an autocomplete deluxe field. Get values from the second element.
          let autocomplete_deluxe_recipients_string = to_recipients_collection[1].value + cc_recipients_collection[1].value;
          const regex = /\((\d+)\)/g;
          const matches = autocomplete_deluxe_recipients_string.match(regex);
          recipients = matches.map((match) => parseInt(match.match(/\d+/)));

        }
        else {
          // Suppose this is a regular autocomplete field:
          let recipients_length = to_recipients_collection.length + cc_recipients_collection.length;
          if (recipients_length > 0) {
            let recipients_collection = to_recipients_collection.concat(cc_recipients_collection);
            recipients_collection.forEach(function (recipient, index) {
              if (index < recipients_length) {
                if (recipient.value.match(/\(\d+\)/)) {
                  let id_parenthesis = recipient.value.match(/\(\d+\)/)[0];
                  recipients.push(id_parenthesis.substring(1, id_parenthesis.length - 1));
                }
              }
            });
          }
        }
      }


      let proc_path_prefix_add = window.location.origin + drupalSettings.path.baseUrl + "proc/add";
      let ids_csv = recipients.join();

      // Concatenate predefined recipients:
      if (pre_set_recipients_csv != "") {
        ids_csv = ids_csv + "," + pre_set_recipients_csv;
      }

      let parent_selector = jQuery(this).parent();
      let parent_id = parent_selector[0].getAttribute("id").slice(0, -13);
      let field_input_mode = {$field_input_mode};

      let proc_path_suffix = "?proc_standalone_mode=FALSE&proc_parent_id=" + parent_id + "&proc_field_name={$proc_field_name}&field-input-mode=" + field_input_mode;

      {$fetcher_value}
      return false;
    JS;

    $direct_fetcher_js = $direct_fetcher_js_prefix . $direct_fetcher_to . $direct_fetcher_cc . $direct_fetcher_js;

    // Create the URL for the Encrypt button.
    $url = Url::fromUserInput('#', [
      'attributes' => [
        'onclick' => $direct_fetcher_js,
        'id' => 'encrypt-' . $proc_field_name,
      ],
    ]);

    // Create the Encrypt button link.
    $encrypt_button_class = [$config->get('proc-encrypt-button-class')];
    if ($trigger_format === '1') {
      $encrypt_button_class = ['hidden'];
    }

    $link = [
      '#title' => $config->get('proc-encrypt-button-label'),
      '#type' => 'link',
      '#url' => $url,
      '#attributes' => [
        'class' => $encrypt_button_class,
      ],
    ];

    // Create the checkbox for encryption.
    $checkbox = [
      '#type' => 'checkbox',
      '#title' => 'Encrypt',
      '#default_value' => [0 => 0],
      '#id' => 'encrypt-checkbox-' . $proc_field_name,
      '#attributes' => [
        'onclick' => <<<JS

            if (this.checked) {
                document.getElementById("encrypt-{$proc_field_name}").click();
                this.value = 1;
            }
            else {
                document.getElementById("decrypt-{$proc_field_name}").click();
                this.value = 0;
            }
        JS,
        'value' => 0,
      ],
    ];

    $element['target_id']['#attached']['library'][] = 'proc/proc-field';

    if ($submit_element_id) {
      $element['target_id']['#attached']['drupalSettings']['proc']['submit_element_id'] = $submit_element_id;
    }

    $decryption_link = [];

    $decryption_mode = FALSE;
    $proc_id = '';
    // If there is a default value, add also the Decrypt button:
    if ($element['target_id']['#default_value']) {

      if (isset($selection_settings['proc']['proc_field_decryption_mode'])) {
        $decryption_mode = $selection_settings['proc']['proc_field_decryption_mode'];
      }

      $proc_id = $element['target_id']['#default_value']->get('id')->getValue()[0]['value'];
      $decryption_url = Url::fromUserInput(
        '/proc/' . $proc_id,
        [
          'attributes' => [
            'class' => ['button'],
          ],
        ]
      );

      $decryption_link = Link::fromTextAndUrl($this->t('Decrypt'), $decryption_url);
      $decryption_link = $decryption_link->toRenderable();

      // If decryption has to happen in a dialog:
      if ($decryption_mode == '1') {
        $dialog_decrypt_js = <<<JS
          let proc_path_prefix_add = window.location.origin + drupalSettings.path.baseUrl + "proc/{$proc_id}";
          let parent_selector = jQuery(this).parent();
          let parent_id = parent_selector[0].getAttribute("id").slice(0, -13);
          let field_input_mode = {$field_input_mode};
          let field_decryption_mode = {$decryption_mode};
          const cache_password = {$cache_password};

          let proc_path_suffix = "?proc_standalone_mode=FALSE&cache-password=" + cache_password + "&proc_parent_id=" + parent_id + "&proc_field_name={$proc_field_name}&field-input-mode=" + field_input_mode + "&field-decryption-mode=" + field_decryption_mode + "&trigger-fields={$trigger_fields}";
          let proc_path = proc_path_prefix_add + "/" + proc_path_suffix;

          jQuery(this).attr("href", "#");

          let ajaxSettings = {
            url: proc_path,
            dialogType: "dialog",
            dialog: { width: 400, classes: { "ui-dialog": "class-{$proc_field_name}" } },
          };
          let myAjaxObject = Drupal.ajax(ajaxSettings);
          myAjaxObject.execute();
          return false;
        JS;

        $dialog_decrypt_url = Url::fromUserInput(
        // URL with recipients IDs will be defined
        // by the direct fetcher.
          '#',
          [
            'attributes' => [
              'onclick' => $dialog_decrypt_js,
              'class' => [$config->get('proc-encrypt-button-class')],
              'id' => 'decrypt-' . $proc_field_name,
            ],
          ]
        );
        $decryption_link['#url'] = $dialog_decrypt_url;
      }
    }

    $description_proc_elements = [$link];
    if ($trigger_format === '1') {
      $description_proc_elements = [
        $link,
        $decryption_link,
        $checkbox,
      ];
    }

    $element['target_id']['#description'] = $description_proc_elements;

    $decryption_link['#attributes']['class'] = 'hidden';

    if ($decryption_link['#type'] == 'link') {
      $checkbox['#attributes']['checked'] = 'checked';
      $element['target_id']['#description'] = [
        $link,
        $decryption_link,
        $checkbox,
      ];
      if ($trigger_format === '0') {
        $decryption_link['#attributes']['class'] = [$config->get('proc-encrypt-button-class')];
        $element['target_id']['#description'] = [$link, $decryption_link];
      }
    }

    // If this is an encrypted text field:
    if ($field_input_mode == 2) {
      $proc_object = FALSE;
      if ($proc_id) {
        $proc_object = Proc::load($proc_id);
      }
      // @todo disable autocomplete:
      // Add hidden field for proc_id:
      $element['target_id']['#description'] = [$element['target_id']['#description']];
      $element['target_id']['#default_value'] = $proc_object ?? '';
    }

    $element['target_id']['#attributes']['proc'] = 'true';

    return $element;
  }

}
